@@ -1,6 +1,7 @@
 #include <Rcpp.h>
 #include "encoding.h"
 #include "parsing.h"
+#include "compose.h"
 
 using namespace Rcpp;
 
@@ -80,7 +81,8 @@ std::vector < std::string > url_encode(std::vector < std::string > urls){
   //Measure size, create output object and holding objects
   int input_size = urls.size();
   std::vector < std::string > output(input_size);
-  size_t indices;
+  size_t scheme_start;
+  size_t first_slash;
   encoding enc_inst;
 
   //For each string..
@@ -92,12 +94,17 @@ std::vector < std::string > url_encode(std::vector < std::string > urls){
     }
     
     //Extract the protocol. If you can't find it, just encode the entire thing.
-    indices = urls[i].find("://");
-    if(indices == std::string::npos){
+    scheme_start = urls[i].find("://");
+    if(scheme_start == std::string::npos){
       output[i] = enc_inst.internal_url_encode(urls[i]);
     } else {
       //Otherwise, split out the protocol and encode !protocol.
-      output[i] = urls[i].substr(0,indices+3) + enc_inst.internal_url_encode(urls[i].substr(indices+3));
+      first_slash = urls[i].find("/", scheme_start+3);
+      if(first_slash == std::string::npos){
+        output[i] = urls[i].substr(0,scheme_start+3) + enc_inst.internal_url_encode(urls[i].substr(scheme_start+3));
+      } else {
+        output[i] = urls[i].substr(0,first_slash+1) + enc_inst.internal_url_encode(urls[i].substr(first_slash+1));
+      }
     }
   }
   
@@ -160,16 +167,15 @@ List url_parameters(std::vector < std::string > urls, std::vector < std::string
 //'perfectly suitable for the intended purpose (decomposition in the context of automated
 //'HTTP requests from R), but not for large-scale analysis.
 //'
-//'@return a list of vectors, one for each URL, with each vector containing (in sequence)
-//'the URL's scheme, domain, port, path, query string and fragment identifier. See the
-//'\href{http://tools.ietf.org/html/rfc3986}{relevant IETF RfC} for definitions. If an element
-//'cannot be identified, it is replaced with an empty string.
+//'@return a data.frame consisting of the columns scheme, domain, port, path, query
+//'and fragment. See the '\href{http://tools.ietf.org/html/rfc3986}{relevant IETF RfC} for
+//'definitions. If an element cannot be identified, it is represented by an empty string.
 //'
 //'@examples
 //'url_parse("https://en.wikipedia.org/wiki/Article")
 //'
-//'@seealso \code{url_parameters} for extracting values associated with particular keys in a URL's
-//'query string.
+//'@seealso \code{\link{url_parameters}} for extracting values associated with particular keys in a URL's
+//'query string, and \code{\link{url_compose}}, which is \code{url_parse} in reverse.
 //'
 //'@export
 //[[Rcpp::export]]
@@ -178,3 +184,29 @@ DataFrame url_parse(std::vector < std::string > urls){
   parsing p_inst;
   return p_inst.parse_to_df(urls_ptr);
 }
+
+//'@title Recompose Parsed URLs
+//'
+//'@description Sometimes you want to take a vector of URLs, parse them, perform
+//'some operations and then rebuild them. \code{url_compose} takes a data.frame produced
+//'by \code{\link{url_parse}} and rebuilds it into a vector of full URLs (or: URLs as full
+//'as the vector initially thrown into url_parse).
+//'
+//'This is currently a `beta` feature; please do report bugs if you find them.
+//'
+//'@param parsed_urls a data.frame sourced from \code{\link{url_parse}}
+//'
+//'@seealso \code{\link{scheme}} and other accessors, which you may want to
+//'run URLs through before composing them to modify individual values.
+//'
+//'@examples
+//'#Parse a URL and compose it
+//'url <- "http://en.wikipedia.org"
+//'url_compose(url_parse(url))
+//'
+//'@export
+//[[Rcpp::export]]
+std::vector < std::string > url_compose(DataFrame parsed_urls){
+  compose c_inst;
+  return c_inst.compose_multiple(parsed_urls);
+}
